var dbApi = {};
dbApi.indexedDB = {};
var indexedDBInitialized = false;

dbApi.indexedDB.db = null;

dbApi.indexedDB.open = function(onsuccess) {
    var version = 1;
    var request = indexedDB.open("log", version);

    // We can only create Object stores in a versionchange transaction.
    request.onupgradeneeded = function(e) {
        var db = e.target.result;

        // A versionchange transaction is started automatically.
        e.target.transaction.onerror = function(e) {
            console.log("Failed to upgrade IndexedDB: " + e)
        };

        e.target.transaction.onabort = function(e) {
            console.log("abort: " + e.target.error.message);
        };

        if(db.objectStoreNames.contains("entries")) {
            db.deleteObjectStore("entries");
        }

        var store = db.createObjectStore("entries", {autoIncrement: true});
        store.createIndex("timeStamp", "timeStamp", {unique: true});
    };

    request.onsuccess = function(e) {
        indexedDBInitialized = true;
        indexedDB.db = e.target.result;
        if (onsuccess != undefined) {
            onsuccess();
        }
    };

    request.onerror = function(e) {
        console.log("Failed to initialize IndexedDB: " + e.target.errorCode);
    };
};

dbApi.indexedDB.logMessage = function(message, type) {
    console.log(message);
    if (indexedDBInitialized == true) {
        var db = indexedDB.db;
        var trans = db.transaction(["entries"], "readwrite");
        var store = trans.objectStore("entries");
        var request = store.put({
            "message": message,
            "type": type,
            "timeStamp": new Date()
        });

        request.onerror = function(e) {
            console.log("Error logging message: " + e.target.errorCode);
        };
    }
};

dbApi.indexedDB.getAllLogEntries = function(entriesTable, dateFrom, dateTo, type) {
    if (indexedDBInitialized == true) {
        entriesTable.clear();

        if(dateFrom.trim() != "")  {
            dateFrom = new Date(dateFrom);
        }

        if(dateTo.trim() != "") {
            dateTo = new Date(dateTo);
        }

        type = type.trim();

        var db = indexedDB.db;
        var trans = db.transaction(["entries"], "readwrite");
        var store = trans.objectStore("entries");
        var index = store.index('timeStamp');

        var keyRange;
        if(dateFrom != "" && dateTo != "") {
            keyRange = IDBKeyRange.bound(dateFrom, dateTo);
        } else if(dateFrom == "") {
            keyRange = IDBKeyRange.upperBound(dateTo);
        } else {
            keyRange = IDBKeyRange.lowerBound(dateFrom);
        }

        var request = index.openCursor(keyRange, "prev");

        request.onsuccess = function (e) {
            var result = e.target.result;
            if (result) {
                if (type == null || type == "all" || type == result.value.type) {
                    renderLogEntry(entriesTable, result.value);
                }
                result.continue();
            } else {
                entriesTable.draw();
            }
        };

        request.onerror = function(e) {
            console.log("Error getting log entries:" + e.target.errorCode);
        };
    }
};

//Delete log entries that are older than 30 days.
dbApi.indexedDB.truncate = function() {
    if (indexedDBInitialized == true) {
        var db = indexedDB.db;
        var trans = db.transaction(["entries"], "readwrite");
        var store = trans.objectStore("entries");
        var index = store.index('timeStamp');

        var date = new Date();
        date.setDate(date.getDate() - 30);

        var keyRange = IDBKeyRange.upperBound(date);
        var request = index.openCursor(keyRange);

        request.onsuccess = function (e) {
            var result = e.target.result;
            if (result) {
                result.delete();
                result.continue();
            }
        };

        request.onerror = function (e) {
            console.log("Error truncating log: " + e.target.errorCode);
        };
    }
};

function renderLogEntry(entriesTable, row) {
    entriesTable.row.add([new Date(row.timeStamp).toLocaleString(), row.message, row.type]);
}


