var lostConnectivity = false;
var noInternetMessage = "Failed to validate license. Check internet connection.  Trying again in 1 minute.";
var successCount = 0;
var waitTime = 60000;

$(document).ready(function() {
    initialize();
});

function initialize() {
    var currentWindow = chrome.app.window.current();

    if(!currentWindow.isFullscreen()) {
        currentWindow.fullscreen();
    }

    loadPage();
}

function loadPage() {
    chrome.storage.local.get('hypersign_APIKEY', function(result) {
        if(result.hypersign_APIKEY) {
            //Validate the license and show the content.
            validateLicenseAndLoadContent(result.hypersign_APIKEY);
            //Validate the license once a day.
            setInterval(function() {validateLicense(result.hypersign_APIKEY)}, ONCE_A_DAY_INTERVAL);
            setTimeout(function() {checkHealth()}, waitTime);

        } else {
            var webview = '<webview style="width:0px;height:0px;border:none;outline:none;" id="map" src="' + baseUrl + 'device/pairingPortal"></webview>';
            $("#pageContainer").prepend(webview);
            $(".pairDeviceForm").show();
            $("#pairingCode").focus();
            document.getElementById("submitCode").onclick = submitCode;
        }
    });
}

function checkHealth() {
    console.log(successCount);
    setTimeout(function() {checkHealth()}, waitTime);
    try {
        $.ajax({
            type: "GET",
            url: baseUrl + "status/ping",
            cache: false,
            crossDomain: true,
            timeout: 59000,
            success: function (msg) {
                //restart if we were previously disconnected
                if (lostConnectivity && successCount > 9) {
                    lostConnectivity = false;
                    //we should reload after 10 minutes
                    setTimeout(function () {
                        try {
                            chrome.runtime.restart();
                        }catch(e){}
                        try {
                            chrome.runtime.reload();
                        }catch(e){}
                    }, 2000);
                }
                //can't allow this to go on indefinitely, or it wou
                if(successCount < 50)
                    successCount++;
            },
            error: function (XMLHttpRequest, textStatus, errorThrown) {
                successCount = 0;
                lostConnectivity = true;
                waitTime = 15000;
            },
            failure: function () {
                successCount = 0;
                lostConnectivity = true;
                waitTime = 15000;
            }
        });
    }
    catch(e){
        lostConnectivity = true;
    }
}

function submitCode() {
    var pairingCode = $("#pairingCode").val().toUpperCase();
    var valid = true;
    var errorMessage = "";
    if (pairingCode.length != 5) {
        valid = false;
        errorMessage = "Pairing Code must be 5 characters long. Please try again.";
    }
    if (valid) {
        var apikey;
        $.ajax({
            url: baseUrl + "device/pair/" + pairingCode,
            type: 'POST',
            dataType: 'json',
            contentType: "application/json; charset=utf-8",
            data: {
                "code": pairingCode
            },
            async: true,
            success: function(data) {
                if (data.success) {
                    apikey = data.key;
                    if (apikey) {
                        chrome.storage.local.set({
                            'hypersign_APIKEY': apikey
                        }, function() {});
                        // chrome.runtime.restart();
                        chrome.runtime.reload();
                    }
                } else {
                    $("#pairingCode").val('');
                    $("#invalidPairingCode").text("Invalid pairing code. Please try again.");
                }
            },
            failure: function(data) {
                logMessage("Failed to pair device.", LOG_TYPE_GENERIC_ERROR);
                $("#invalidPairingCode").text("Process failed. Please try again.");
            }
        });
    } else {
        //not valid
        $("#invalidPairingCode").text(errorMessage);
    }
}

function validateLicenseAndLoadContent(apiKey) {
    validateLicense(apiKey, loadContent);
}

function loadContent(apiKey) {
    var webview = '<webview id="map" src="' + baseUrl + 'device/show/' + apiKey + '" style="padding:0;margin:0;height:100%;width:100%;border:none;outline:none;"></webview>';
    $("#pageContainer").html(webview);
}

function validateLicense(apiKey, successCallback) {
    $.ajax({
        url: baseUrl + "device/licenseInformation/" + apiKey,
        type: 'GET',
        dataType: 'json',
        timeout: 600000,
        success: function (data) {
            //If the license is valid, call the callback function (if provided).
            if (data.licenseInformation.valid) {
                if (successCallback) {
                    successCallback(apiKey)
                }
            } else {
                logMessage("The license is not valid.", LOG_TYPE_INVALID_LICENSE);
                chrome.app.window.create("invalidLicense.html", {
                        id: "InvalidLicense",
                        state: "fullscreen",
                        resizable: false,
                        alwaysOnTop: true,
                        focused: true
                    }
                );
                setTimeout(function(){
                    // chrome.runtime.restart();
                    chrome.runtime.reload();
                },60000);
            }
        },
        failure: function (msg) {
            //logMessage("Failed to validate the license.", LOG_TYPE_GENERIC_ERROR);
            $("#invalidPairingCode").text("Fail: " + noInternetMessage);
            setTimeout(function(){
                // chrome.runtime.restart();
                chrome.runtime.reload();
            },60000);
        },
        error: function (msg) {
            //logMessage("Failed to validate the license.", LOG_TYPE_GENERIC_ERROR);
            $("#invalidPairingCode").text("Error: " + noInternetMessage);
            setTimeout(function(){
                // chrome.runtime.restart();
                chrome.runtime.reload();
            },60000);
        }
    });
}